/*
 *      pam_tcfs.so v1.0.1
 *
 *      written by Luigi Catuogno [luicat@mikonos.dia.unisa.it]
 *
 *      Jul 2 1997
 */

#include <stdio.h>
#include <linux/fs.h>
#include <linux/tcfs_fs.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <mntent.h>
#include <errno.h>
#include <pwd.h>
#include <sys/types.h>
#include <syslog.h>

#include "linux/kdes.h"
#include "defines.h"
#include "tcfscode.h"


/*	funzioni di supporto di tcfs
*/

int tcfs_enable(char *u,char *tcfskey)
{
    struct login user;
    int fd,set=0;
    struct mntent *mount_points;
    FILE *mtab;

    strncpy (user.deskey, tcfskey, KEYSIZE);
    user.uid = (getpwnam(u)->pw_uid);


    /* Search for TCFS mount points */
    if ( ( mtab = setmntent("/etc/mtab", "r")) == NULL){
	perror("tcfslogin");
	closelog();
	return 0;
    }

    while( ( mount_points = getmntent(mtab) ) != NULL ){


	if ( strcmp(mount_points->mnt_type,"tcfs") == 0 ) {
	  /* Got a TCFS File System */
	    if( ( fd = open(mount_points->mnt_dir,O_RDONLY)) < 0 ){
		perror (mount_points->mnt_dir);
	    } else {


	      ioctl (fd, TCFS_IOC_LOGIN, &user);
	      set++;
	     }
	  close(fd);
	}
    }
    endmntent(mtab);

    if ( set == 0 ){
	syslog(LOG_WARNING,
	       "Not tcfs filesystems mounted");
	return 0;
    } else 
        return 1;
    
}

#define PASS_LENGTH  128

int  tcfs_key (char *u, char *pwd, char *t, char **tk)
{
    int len,i=0;
    char pass[PASS_LENGTH],*cypher;
    static char *tcfskey;
    des_key_schedule ks;

    strncpy(pass,pwd,PASS_LENGTH);
  
    
    tcfskey=tcfs_decode(t, &len);
  
    while( strlen(pass) < 8 ){
	char tmp[128];
	strcpy(tmp,pass);
	strcpy(tmp,pass);
	strcpy(pass,tmp);
    }  

    while((i*8)<=KEYSIZE)
	{
    	 des_set_key ((des_cblock *)pass, ks);

         des_ecb_encrypt ((des_cblock *)(tcfskey+i*8), 
             (des_cblock *)(tcfskey+i*8), ks, DES_DECRYPT);
	 i++;
	}
    	 bzero (pass,strlen(pass));

    memcpy(*tk,tcfskey,KEYSIZE);
    return 0;
}

int tcfs_disable (char *u)
{
    int fd,user,set=0;
    struct mntent *mount_points;
    FILE *mtab;

    user = (getpwnam(u)->pw_uid);

  /* Get TCFS mount points */
    if ( ( mtab = setmntent("/etc/mtab", "r") ) == NULL ){
	perror("tcfslogout");
	exit(1);
    }

    while( ( mount_points = getmntent(mtab) ) != NULL ){


	if( strcmp(mount_points->mnt_type,"tcfs") == 0 ){
	    /* Got a TCFS File System */
	    if( ( fd = open(mount_points->mnt_dir,O_RDONLY) ) < 0 ){
		perror(mount_points->mnt_dir);
	    } else {


		ioctl (fd, TCFS_IOC_LOGOUT, &user);
		set++;
	    }
	    close (fd);
	}
    }
    endmntent (mtab);

    if ( set == 0 ){
	syslog(LOG_WARNING,
	       "no tcfs filesystems mounted");
	return 0;
    } else 
	return 1;
    
 return 1;
}


